package com.diagbot.facade;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.aggregate.AverageStatisticsAggregate;
import com.diagbot.aggregate.MrStatisticsAggregate;
import com.diagbot.aggregate.ResultStatisticsAggregate;
import com.diagbot.dto.AverageStatisticsDTO;
import com.diagbot.dto.NumDTO;
import com.diagbot.dto.QcResultPercentDTO;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.vo.FilterPageVO;
import com.diagbot.vo.FilterVO;
import com.diagbot.vo.QcresultFilterVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.aggregate.facade.DataBeanAggregateQueryFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2020/4/13 16:45
 */
@Component
public class ConsoleFacade {
    @Autowired
    private FilterFacade filterFacade;
    @Autowired
    private DataBeanAggregateQueryFacade dataBeanAggregateQueryFacade;

    @Autowired
    private AverageStatisticsAggregate averageStatisticsAggregate;
    @Autowired
    private MrStatisticsAggregate mrStatisticsAggregate;
    @Autowired
    private ResultStatisticsAggregate resultStatisticsAggregate;
    @Autowired
    private QcresultInfoFacade qcresultInfoFacade;
    @Autowired
    private QcCasesFacade qcCasesFacade;
    @Autowired
    private HomePageFacade homePageFacade;
    @Autowired
    private BehospitalInfoFacade behospitalInfoFacade;

    //-----------------------聚合接口开始-------------------------------

    /**
     * 病历相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllMr", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 缺陷相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> resultStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllResult", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    /**
     * 平局值相关统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> averageStatistics(FilterVO filterVO) {
        Map<String, Object> retMap = new LinkedHashMap<>();
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setHospitalId(hospitalId);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
        try {
            Map<String, Object> invokeParams = new HashMap<>();
            invokeParams.put("filterVO", filterVO);
            retMap
                    = dataBeanAggregateQueryFacade.get("setAllAverage", invokeParams, Map.class);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR);
        }
        return retMap;
    }

    //-----------------------聚合接口结束-------------------------------

    //-----------------------单独接口开始-------------------------------

    /**
     * 平均住院天数
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageDayNum(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageDayNumList = averageStatisticsAggregate.getAverageDayNum(filterVO);
        if (ListUtil.isNotEmpty(averageDayNumList)) {
            retMap.put("平均住院日", averageDayNumList);
        }
        return retMap;
    }

    /**
     * 平均住院费用
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageFee(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageFeeList = averageStatisticsAggregate.getAverageFee(filterVO);
        if (ListUtil.isNotEmpty(averageFeeList)) {
            retMap.put("平均住院费用", averageFeeList);
        }
        return retMap;
    }

    /**
     * 质控平均分按科室统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getAverageScore(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<AverageStatisticsDTO> averageScoreList = averageStatisticsAggregate.getAverageScore(filterVO);
        if (ListUtil.isNotEmpty(averageScoreList)) {
            retMap.put("各科室质控平均分", averageScoreList);
        }
        return retMap;
    }

    /**
     * 各科室甲级病历占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> getLevelResultDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<QcResultPercentDTO> levelResults = mrStatisticsAggregate.getLevelResultDept(filterVO);
        if (ListUtil.isNotEmpty(levelResults)) {
            retMap.put("各科室甲级病历占比", levelResults);
        }
        return retMap;
    }

    /**
     * 出院人数统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> leaveHosCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        Map<String, Object> leaveHosMap = mrStatisticsAggregate.leaveHosCount(filterVO);
        if (leaveHosMap != null) {
            retMap.put("出院人数统计", leaveHosMap);
        }
        return retMap;
    }

    /**
     * 质控病历统计
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> mrCount(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> mrCountList = mrStatisticsAggregate.mrCount(filterVO);
        if (ListUtil.isNotEmpty(mrCountList)) {
            retMap.put("病历数统计", mrCountList);
        }
        return retMap;
    }

    /**
     * 各模块缺陷占比排行
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByCase(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> caseList = resultStatisticsAggregate.entryCountGroupByCase(filterVO);
        if (ListUtil.isNotEmpty(caseList)) {
            retMap.put("各模块缺陷占比排行", caseList);
        }
        return retMap;
    }

    /**
     * 条目缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryCountGroupByEntry(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> entryList = resultStatisticsAggregate.entryCountGroupByEntry(filterVO);
        if (ListUtil.isNotEmpty(entryList)) {
            retMap.put("条目缺陷占比", entryList);
        }
        return retMap;
    }

    /**
     * 各科室缺陷占比
     *
     * @param filterVO
     * @return
     */
    public Map<String, Object> entryByDept(FilterVO filterVO) {
        Map<String, Object> retMap = new HashMap<>();
        filterVOSet(filterVO);
        List<NumDTO> deptList = resultStatisticsAggregate.entryByDept(filterVO);
        if (ListUtil.isNotEmpty(deptList)) {
            retMap.put("各科室缺陷占比", deptList);
        }
        return retMap;
    }

    private void filterVOSet(FilterVO filterVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = filterFacade.getStartDateStr(filterVO.getType(), null);
        String endDate = filterFacade.getEndDateStr(filterVO.getType(), null);
        filterVO.setStartDate(startDate);
        filterVO.setEndDate(endDate);
        filterVO.setHospitalId(hospitalId);
        if (filterVO.getLimitCount() == null || filterVO.getLimitCount().equals(0)) {
            filterVO.setLimitCount(10);
        }
    }
    //-----------------------单独接口结束-------------------------------

    //_______________________分页接口开始-------------------------------

    /**
     * 过滤条件设置
     *
     * @param filterPageVO
     */
    private void filterPageVOSet(FilterPageVO filterPageVO) {
        String hospitalId = SysUserUtils.getCurrentHospitalID();
        String startDate = "";
        String endDate = "";
        Date date = new Date();

        //1-本月，2-本年，3-上月，4-去年本月，5-去年
        if (filterPageVO.getType().equals(1) || filterPageVO.getType().equals(2)) {
            startDate = filterFacade.getStartDateStr(filterPageVO.getType(), null);
            endDate = filterFacade.getEndDateStr(filterPageVO.getType(), null);
        } else if (filterPageVO.getType().equals(3) || filterPageVO.getType().equals(5)) {
            startDate = filterFacade.getLastStartDateStr(filterPageVO.getType());
            endDate = filterFacade.getLastEndDateStr(filterPageVO.getType());
        } else if (filterPageVO.getType().equals(4)) {
            String year = DateUtil.getYear(date);
            startDate = filterFacade.getStartDateStr(filterPageVO.getType(), Integer.valueOf(year) - 1);
            endDate = filterFacade.getEndDateStr(filterPageVO.getType(), Integer.valueOf(year) - 1);
        }

        filterPageVO.setStartDate(startDate);
        filterPageVO.setEndDate(endDate);
        filterPageVO.setHospitalId(hospitalId);
    }

    /**
     * 各模块缺陷占比（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> entryCountGroupByCasePage(FilterPageVO filterPageVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        filterPageVOSet(filterPageVO);
        long current = filterPageVO.getCurrent();
        long size = filterPageVO.getSize();
        filterPageVO.setCurrent(0);
        filterPageVO.setSize(filterPageVO.getTotal());
        IPage<NumDTO> page = qcresultInfoFacade.entryCountGroupByCasePage(filterPageVO);
        List<NumDTO> caseList = page.getRecords();
        QcresultFilterVO qcresultFilterVO = new QcresultFilterVO();
        BeanUtil.copyProperties(filterPageVO, qcresultFilterVO);
        int mrNum = qcresultInfoFacade.resultCount(qcresultFilterVO);
        List<NumDTO> standardCaseList = qcCasesFacade.entryGroupByCase();
        if (ListUtil.isEmpty(standardCaseList)) {
            return null;
        }
        Map<String, NumDTO> standardCaseMap
                = EntityUtil.makeEntityMap(standardCaseList, "name");
        if (ListUtil.isNotEmpty(caseList)) {
            caseList.forEach(item -> {
                if (!standardCaseMap.containsKey(item.getName())) {
                    item.setTotleNum(0);
                    item.setPercent(0d);
                    item.setPercentStr("0%");
                } else {
                    Integer totleNum = standardCaseMap.get(item.getName()).getNum() * mrNum;
                    item.setTotleNum(totleNum);
                    Double percent = BigDecimal.valueOf(item.getNum())
                            .divide(BigDecimal.valueOf(totleNum), 4, RoundingMode.HALF_UP)
                            .doubleValue();
                    String percentStr
                            = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
                    item.setPercent(percent);
                    item.setPercentStr(percentStr);
                }
            });
            //降序排序
            Collections.sort(caseList, new Comparator<NumDTO>() {
                @Override
                public int compare(NumDTO o1, NumDTO o2) {
                    return o2.getPercent().compareTo(o1.getPercent());
                }
            });
        }
        List<NumDTO> retList = Lists.newLinkedList();
        if (current * size + size > caseList.size()) {
            retList = caseList.subList(Long.valueOf(current * size).intValue(), caseList.size());
        } else {
            retList = caseList.subList(Long.valueOf(current * size).intValue(), Long.valueOf(current * size + size).intValue());
        }
        page.setCurrent(current);
        page.setSize(size);
        page.setRecords(retList);
        return page;
    }

    /**
     * 条目缺陷占比（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> entryCountGroupByEntryPage(FilterPageVO filterPageVO) {
        DecimalFormat df = new DecimalFormat("#0.00");
        filterPageVOSet(filterPageVO);
        IPage<NumDTO> page = qcresultInfoFacade.entryCountGroupByEntryPage(filterPageVO);
        List<NumDTO> records = page.getRecords();
        if (ListUtil.isNotEmpty(records)) {
            records.forEach(item -> {
                Double percent = BigDecimal.valueOf(item.getNum())
                        .divide(BigDecimal.valueOf(item.getTotleNum()), 4, RoundingMode.HALF_UP)
                        .doubleValue();
                item.setPercent(percent);
                String percentStr
                        = df.format(BigDecimal.valueOf(percent).multiply(BigDecimal.valueOf(100))) + "%";
                item.setPercentStr(percentStr);
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 按科室统计平均住院天数（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageDayNumPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<AverageStatisticsDTO> page = homePageFacade.getAverageDayNumPage(filterPageVO);
        return page;
    }

    /**
     * 按科室统计平均住院费用（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageFeePage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<AverageStatisticsDTO> page = homePageFacade.getAverageFeePage(filterPageVO);
        return page;
    }

    /**
     * 各科室质控平均分（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<AverageStatisticsDTO> getAverageScoreByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<AverageStatisticsDTO> page = qcresultInfoFacade.getAverageScoreByDeptPage(filterPageVO);
        return page;
    }

    /**
     * 各科室缺陷占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<NumDTO> resultStatisticsByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<NumDTO> page = behospitalInfoFacade.resultStatisticsByDeptPage(filterPageVO);
        return page;
    }

    /**
     * 各科室甲级病历占比排行（分页）
     *
     * @param filterPageVO
     * @return
     */
    public IPage<QcResultPercentDTO> levelPercentGroupByDeptPage(FilterPageVO filterPageVO) {
        filterPageVOSet(filterPageVO);
        IPage<QcResultPercentDTO> page = qcresultInfoFacade.levelPercentGroupByDeptPage(filterPageVO);
        return page;
    }
    //-----------------------分页接口结束-------------------------------
}